package com.googlecode.jpattern.core;

import java.util.HashMap;
import java.util.Map;

import com.googlecode.jpattern.core.command.ICommand;
import com.googlecode.jpattern.service.log.ALoggerServiceBuilder;
import com.googlecode.jpattern.service.log.ILoggerService;
import com.googlecode.jpattern.service.log.NullLoggerService;
import com.googlecode.jpattern.service.mail.AMailServiceBuilder;
import com.googlecode.jpattern.service.mail.IMailService;
import com.googlecode.jpattern.service.mail.NullMailService;
import com.googlecode.jpattern.service.transaction.ATransactionServiceBuilder;
import com.googlecode.jpattern.service.transaction.ITransactionService;
import com.googlecode.jpattern.service.transaction.NullTransactionService;


/**
 * 
 * @author Francesco Cina'
 *
 * 29/gen/2011
 */
public class ApplicationService implements IProvider , ISystem {

    private static final long serialVersionUID = 1L;
    private Map<String, IService> _mapServices = new HashMap<String, IService>();

    public ApplicationService() {};

    @Override
    public void buildService(AServiceBuilder aServiceBuilder) {
        _mapServices.put(aServiceBuilder.getName(), aServiceBuilder.build());
    }
    
	@Override
	public void buildLoggerService(ALoggerServiceBuilder aLoggerServiceBuilder) {
		buildService(aLoggerServiceBuilder);
	}

	@Override
	public void buildMailService(AMailServiceBuilder aMailServiceBuilder) {
		buildService(aMailServiceBuilder);
	}
	
	@Override
	public void buildTransactionService(ATransactionServiceBuilder aTransactionServiceBuilder) {
		buildService(aTransactionServiceBuilder);
	}
	
    @Override
    public void remove(String aServiceName) {
        _mapServices.remove(aServiceName);
    }    
    
    @Override
    public boolean contains(String aServiceName) {
        return _mapServices.containsKey(aServiceName);
    }        

    @Override
    public IService getService(String aServiceName) {
        if (!contains(aServiceName)) {
            return new NullService();
        }
        return (IService) _mapServices.get(aServiceName);
    }

    @Override
    public void accept(ICommand aCommand) {
        aCommand.visit(  this );
    }

    @Override
    public void removeAllServices() {
        _mapServices.clear();
    }

    @Override
	public ILoggerService getLoggerService() {
		IService service = getService(IServicesName.LOGGER_SERVICE);
		if (service instanceof ILoggerService) {
			return (ILoggerService) service;
		}
		return new NullLoggerService();
	}

	@Override
	public IMailService getMailService() {
		IService service = getService(IServicesName.MAIL_SENDER_SERVICE);
		if (service instanceof IMailService) {
			return (IMailService) service;
		}
		return new NullMailService();
	}

	@Override
	public ITransactionService getTransactionService() {
		IService service = getService(IServicesName.TRANSACTION_SERVICE);
		if (service instanceof ITransactionService) {
			return (ITransactionService) service;
		}
		return new NullTransactionService();
	}

}
