package com.googlecode.jpattern.core.commandfacade;

import java.io.Serializable;

import com.googlecode.jpattern.core.IProvider;
import com.googlecode.jpattern.core.NullProvider;
import com.googlecode.jpattern.core.SystemProxy;
import com.googlecode.jpattern.core.command.ICommand;
import com.googlecode.jpattern.service.log.ILogger;
import com.googlecode.jpattern.service.transaction.ATransactionManager;
import com.googlecode.jpattern.service.transaction.NullTransactionManager;

/**
 * 
 * @author Francesco Cina'
 *
 * 25 Feb 2011
 */
public abstract class ACommandFacade<E> implements Serializable {

    private static final long serialVersionUID = 1L;

    private IProvider provider;
	private ATransactionManager transactionManager = new NullTransactionManager();

	private ILogger logger;

	public ACommandFacade() {
		this(SystemProxy.proxy.getProvider());
	}
	
	public ACommandFacade(IProvider provider) {
		this.provider = provider;
	}
	
    public final synchronized void setTransactionManager(String transactionManagerName, boolean readOnly) {
		transactionManager = getProvider().getTransactionService().getTransactionManager(transactionManagerName);
		transactionManager.setReadOnly(readOnly);
    }
    
    public final synchronized void setTransactionManager(ATransactionManager transactionManager, boolean readOnly) {
		if (transactionManager!=null) {
			this.transactionManager = transactionManager;
			this.transactionManager.setReadOnly(readOnly);
		}
    }

    protected final synchronized void accept(final ICommand aCommand) {
        aCommand.visit(provider);
    }
    
    public final synchronized ICommandFacadeResult<E> exec() {
    	transactionManager.beginTransaction(getProvider());
    	ICommandFacadeResult<E> result = execute();
    	transactionManager.closeTransaction(getProvider(), result);
    	return result;
    }
    
    private synchronized IProvider getProvider() {
    	if (provider==null) {
    		provider = new NullProvider();
    	}
    	return provider;
    }
    
    protected abstract ICommandFacadeResult<E> execute();
    
	protected final ILogger getLogger() {
		if (logger==null) {
			logger = getProvider().getLoggerService().logger(this.getClass());
		}
		return logger;
	}
}
