package com.googlecode.jpattern.service.transaction;

import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import com.googlecode.jpattern.core.IProvider;

/**
 * 
 * @author Francesco Cina'
 *
 * 28 Mar 2011
 * 
 * Manage of the transaction using the PlatformTransactionManager implementation
 * of Spring
 */
public class SpringTransactionManager extends ATransactionManager {

	private boolean readOnly = false;

	private final PlatformTransactionManager platformTransactionManager;

	private TransactionStatus status;

	public SpringTransactionManager(PlatformTransactionManager platformTransactionManager) {
		this.platformTransactionManager = platformTransactionManager;
	}

	@Override
	public void setReadOnly(boolean readOnly) {
		this.readOnly = readOnly;
	}

	@Override
	public void beginTransaction(IProvider provider) {
		DefaultTransactionDefinition def = new DefaultTransactionDefinition();
		def.setReadOnly(readOnly);
		status = platformTransactionManager.getTransaction(def);
		
	}

	@Override
	protected void commit(IProvider provider) {
		if (status!=null) {
			platformTransactionManager.commit(status);
		}
	}

	@Override
	protected void rollback(IProvider provider) {
		if (status!=null) {
			platformTransactionManager.rollback(status);
		}
	}
}
